@@ -1,13 +1,13 @@
 package POE::Component::IKC::Server;
 
 ############################################################
-# $Id: Server.pm 322 2008-01-16 16:40:45Z fil $
+# $Id: Server.pm 819 2011-08-27 01:57:36Z fil $
 # Based on refserver.perl and preforkedserver.perl
 # Contributed by Artur Bergman <artur@vogon-solutions.com>
 # Revised for 0.06 by Rocco Caputo <troc@netrus.net>
 # Turned into a module by Philp Gwyn <fil@pied.nu>
 #
-# Copyright 1999-2008 Philip Gwyn.  All rights reserved.
+# Copyright 1999-2011 Philip Gwyn.  All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
 #
@@ -27,7 +27,7 @@ require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_server);
-$VERSION = '0.2001';
+$VERSION = '0.2302';
 
 sub DEBUG { 0 }
 sub DEBUG_USR2 { 1 }
@@ -45,22 +45,25 @@ BEGIN {
 # This is just a convenient way to create servers.  To be useful in
 # multi-server situations, it probably should accept a bind address
 # and port.
-sub create_ikc_server
+sub spawn
 {
-    my(%params)=@_;
-    $params{package}||=__PACKAGE__;
+    my($package, %params)=@_;
+    $params{package} ||= $package;
 
     unless($params{unix}) {
         $params{ip}||='0.0.0.0';            # INET_ANY
-        $params{port}||=603;                # POE! (almost :)
+        $params{port} = 603                 # POE! (almost :)
+                    unless defined $params{port};
     }
+    $params{protocol} ||= 'IKC0';
 
-    create_ikc_responder();
-    POE::Session->create(
+    # Make sure one is available
+    POE::Component::IKC::Responder->spawn();
+    my $session = POE::Session->create(
                     package_states => [ 
                         $params{package} =>
                         [qw(
-                            _start _stop error
+                            _start _stop error _child
                             accept fork retry waste_time
                             babysit rogues shutdown
                             sig_CHLD sig_INT sig_USR2 sig_USR1 sig_TERM
@@ -68,66 +71,33 @@ sub create_ikc_server
                     ],
                     args=>[\%params],
                   );
+    my $heap = $session->get_heap;
+    return $heap->{wheel_port};
 }
 
-sub spawn
+sub create_ikc_server
 {
-    my($package, %params)=@_;
-    $params{package}=$package;
-    create_ikc_server(%params);
+    my( %params )=@_;
+    $params{package} ||= __PACKAGE__;
+    return $params{package}->spawn( %params );
 }
 
 #----------------------------------------------------
-# NOTE : THIS IS POORLY BEHAVED CODE
 sub _select_define
 {
     my($heap, $on)=@_;
+    return unless $heap->{wheel};
     $on||=0;
 
     DEBUG and 
         warn "_select_define (on=$on)";
 
-    if($POE::VERSION >= 0.24) {
-        if($on) {
-            $heap->{wheel}->resume_accept
-        }
-        else {
-            $heap->{wheel}->pause_accept
-        }
-        return;
-    }
-
-    # NOTE : FOR OLDER VERSIONS OF POE
-    # THIS IS POORLY BEHAVED CODE
-    my $state;
-    my $err="possible redefinition of POE internals";
-    $err="failure to bind to port" if $POE::VERSION <= 0.1005;
-
     if($on) {
-        if(ref $heap->{wheel} eq 'HASH') {
-            $state=$heap->{wheel}->{'state_accept'};
-        } else {
-            $state=$heap->{wheel}->[5];
-        }
-
-        unless(defined $state) {
-            die "No 'state_accept' in $heap->{wheel}, $err.\n";
-        }
-    }
-
-    my $c=0;
-    if(ref $heap->{wheel} eq 'HASH') {
-        foreach my $hndl (qw(socket_handle)) {
-            next unless defined $heap->{wheel}->{$hndl};
-            $poe_kernel->select_read($heap->{wheel}->{$hndl}, $state);
-            $c++;
-        }
+        $heap->{wheel}->resume_accept
     }
     else {
-        $poe_kernel->select_read($heap->{wheel}->[0], $state);
-        $c++;
+        $heap->{wheel}->pause_accept
     }
-    die "No socket_handle in $heap->{wheel}, $err.\n" unless $c;
     return;
 }
 
@@ -142,6 +112,38 @@ sub _delete_wheel
     return;
 }
 
+#----------------------------------------------------
+#
+sub _concurrency_up
+{
+    my( $heap ) = @_;
+    $heap->{concur_connections}++;
+    DEBUG and 
+        warn "$$: $heap->{concur_connections} concurrent connections";
+    return unless $heap->{concurrency} > 0;
+    if( $heap->{concur_connections} >= $heap->{concurrency} ) {
+        DEBUG and 
+            warn "$$: Blocking more concurrency";
+        $heap->{blocked} = 1;
+        _select_define( $heap, 0 );
+    }
+}
+
+
+sub _concurrency_down
+{
+    my( $heap ) = @_;
+    $heap->{concur_connections}--;
+    DEBUG and 
+        warn "$$: $heap->{concur_connections} concurrent connections";
+    return unless $heap->{concurrency} > 0;
+    if( $heap->{concur_connections} < $heap->{concurrency} 
+        and delete $heap->{blocked} ) {
+        DEBUG and 
+            warn "$$: Unblocking concurrency";
+        _select_define( $heap, 1 );
+    }
+}
 
 #----------------------------------------------------
 # Delete all delays
@@ -164,6 +166,8 @@ sub _start
 {
     my($heap, $params, $kernel) = @_[HEAP, ARG0, KERNEL];
 
+    my $ret;
+
     # monitor for shutdown events.
     # this is the best way to get IKC::Responder to tell us about the
     # shutdown
@@ -184,19 +188,31 @@ sub _start
     }
     else {
         $alias="$params->{ip}:$params->{port}";
-        $wheel_p{BindPort}= $params->{port};
+        $wheel_p{BindPort} = $params->{port};
         $wheel_p{BindAddress}= $params->{ip};
     }
     DEBUG && warn "$$: Server starting $alias.\n";
 
-    # +GC
-    $kernel->alias_set("IKC Server $alias");
 
-                                        # create a socket factory
-    $heap->{wheel_address}=$alias;
-    $heap->{wheel} = new POE::Wheel::SocketFactory (%wheel_p);
     $heap->{name}=$params->{name};
     $heap->{kernel_aliases}=$params->{aliases};
+    $heap->{concurrency}=$params->{concurrency} || 0;
+    $heap->{protocol}=$params->{protocol};
+                                        # create a socket factory
+    $heap->{wheel} = new POE::Wheel::SocketFactory (%wheel_p);
+    if( $heap->{wheel} and not $params->{unix} and not $params->{port} ) {
+        $heap->{wheel_port} = 
+                $ret = ( sockaddr_in( $heap->{wheel}->getsockname() ) )[0];
+        $alias="$params->{ip}:$ret";
+        DEBUG && 
+                warn "$$: Server listening on $alias.\n";
+    }
+    $heap->{wheel_address}=$alias;
+
+    $heap->{connections} = 0;
+
+    # +GC
+    $kernel->alias_set("IKC Server $alias");
 
     # set up local names for kernel
     my @names=($heap->{name});
@@ -210,7 +226,16 @@ sub _start
 
     $kernel->post(IKC=>'register_local', \@names);
 
-    return unless $params->{processes};
+    # pre-load the default serialisers
+    foreach my $ft ( qw(Storable FreezeThaw POE::Component::IKC::Freezer) ) {
+        eval {  local $SIG{__WARN__} = sub {1}; 
+                local $SIG{__DIE__} = 'DEFAULT';
+                POE::Filter::Reference->new( $ft );
+             };
+        warn "$ft: $@" if DEBUG and $@;
+    }
+
+    return $ret unless $params->{processes};
 
     # Delete the SocketFactory's read select in the parent
     # We don't ever want the parent to accept a connection
@@ -249,7 +274,47 @@ sub _start
         };
         $kernel->yield('babysit');
     }
-    return;
+    return $ret;
+}
+
+#------------------------------------------------------------------------------
+sub _child
+{
+    my( $heap, $kernel, $op, $child, $ret ) = 
+                                @_[ HEAP, KERNEL, ARG0, ARG1, ARG2 ];
+    $ret ||= '';
+    DEBUG and 
+        warn "$$: _child op=$op child=$child ret=$ret";
+    unless( $ret eq "channel-$child" ) {
+        if( $op eq 'create' ) {
+            DEBUG and 
+                warn "$$: Detatching child session $child";
+            $kernel->detach_child( $child );
+        }
+        return;
+    }
+    if( $op eq 'lose' ) {
+        DB::disable_profile() if $INC{'Devel/NYTProf.pm'};
+        $heap->{child_sessions}--;
+        if( $heap->{child_sessions} > 0 ) {
+            DEBUG and warn "$$: still have a child session";
+        }
+        _concurrency_down($heap);
+    }
+    else {
+        $heap->{child_sessions}++;
+        return;
+    }
+    unless( $heap->{wheel} ) {  # no wheel == GAME OVER
+        # DEBUG and 
+        $INC{'Test/More.pm'} or 
+            warn "$$: }}}}}}}}}}}}}}} Game over\n";
+        # XXX: Using shutdown is a stop-gap measure.  Maybe the daemon
+        # wants to stay alive even if IKC was shutdown...
+        # XXX: more to the point, maybe there are still requests that are
+        # hanging around !
+        $kernel->call( IKC => 'shutdown' );
+    }
 }
 
 #------------------------------------------------------------------------------
@@ -312,9 +377,9 @@ sub babysit
                 $time=eval{$table{$pid}->utime + $table{$pid}->stime};
                 warn $@ if $@;
                 # utime and stime are Linux-only :(
-                $time /= 1_000_000 if $time;    # in micro-seconds
+                $time /= 1_000_000 if $time;    # micro-seconds -> seconds
 
-                if($time and $time > 600) { # arbitrary limit of 10 minutes
+                if($time and $time > 1200) {    # arbitrary limit of 20 minutes
                     $rogues{$pid}=$table{$pid};
                         warn "$$: $pid has gone rogue, time=$time s\n";
                 } else {
@@ -462,7 +527,7 @@ sub shutdown
     _delete_delays();               # get it OVER with
 
     # -GC
-    $kernel->alias_remove("IKC Server $heap->{wheel_address}");
+    # $kernel->alias_remove("IKC Server $heap->{wheel_address}");
     $heap->{'die'}=1;               # prevent race conditions
 }
 
@@ -501,6 +566,7 @@ sub accept
     my ($heap, $kernel, $handle, $peer_host, $peer_port) = 
             @_[HEAP, KERNEL, ARG0, ARG1, ARG2];
 
+    T->start( 'IKC' );
     if(DEBUG) {
         if($peer_port) {        
             warn "$$: Server connection from ", inet_ntoa($peer_host), 
@@ -515,33 +581,37 @@ sub accept
     }
     if($heap->{children} and not $heap->{'is a child'}) {
         warn "$$: Parent process received a connection: THIS SUCKS\n";
-        _select_define($heap, 0);
         return;
     }
 
+    DB::enable_profile() if $INC{'Devel/NYTProf.pm'};
+
     DEBUG and warn "$$: Server kernel_aliases=", join ',', @{$heap->{kernel_aliases}||[]};
 
                                         # give the connection to a channel
     POE::Component::IKC::Channel->spawn(
-                handle=>$handle, name=>$heap->{name},
-                unix=>$heap->{unix}, aliases=>[@{$heap->{kernel_aliases}||[]}]);
+                handle=>$handle, 
+                name=>$heap->{name},
+                unix=>$heap->{unix}, 
+                aliases=>[@{$heap->{kernel_aliases}||[]}],
+                protocol=>$heap->{protocol}
+            );
+
+    _concurrency_up($heap);
         
     return unless $heap->{children};
 
-    if ($heap->{'is a child'}) {
+    if (--$heap->{connections} < 1) {
+        # DEBUG and 
+                warn "$$: {{{{{{{{{{{{{{{ Game over\n";
+        $kernel->delay('waste_time');
+        _delete_wheel( $heap );
+        $::TRACE_REFCNT = 1;
 
-        if (--$heap->{connections} < 1) {
-            # DEBUG and 
-                warn "$$: ************* Game over\n";
-            $kernel->delay('waste_time');
-            _delete_wheel( $heap );
-
-        } else {
-            # DEBUG and 
-                warn "$$: $heap->{connections} connections left\n";
-        }
     } else {
-        warn "$$: Master client got a connect!  this sucks!\n";
+        # DEBUG and 
+                warn "$$: $heap->{connections} connections left\n";
+        _select_define($heap, 0);
     }
 }
 
@@ -592,8 +662,17 @@ sub fork
                                         # child becomes a child server
     else {
         $heap->{verbose} and warn "$$: Created ", scalar localtime, "\n";
-        # Clean out stuff that the parent needs but not the children
 
+        # This resets some kernel data that was preventing the child process's
+        # kernel from becoming IDLE
+        if( $kernel->can( 'has_forked' ) ) {
+            $kernel->has_forked;
+        }
+        else {
+            $kernel->_data_sig_initialize;
+        }
+
+        # Clean out stuff that the parent needs but not the children
         $heap->{'is a child'}   = 1;        # don't allow fork
         $heap->{'failed forks'} = 0;
         $heap->{children}={};               # don't kill child processes
@@ -763,20 +842,22 @@ sub __peek
 {
     my($verbose)=@_;
     eval {
-        require POE::API::Peek;
         require POE::Component::Daemon;
     };
+    unless( $@ ) {
+        my $ret = Daemon->peek( $verbose );
+        $ret =~ s/\n/\n$$: /g;
+        warn "$$: $ret";
+        return 1;
+    }
+
+    eval {
+        require POE::API::Peek;
+    };
     if($@) {
         DEBUG and warn "Failed to load POE::API::Peek: $@";
         return;
     }
-    my $ret = Daemon->peek( $verbose );
-
-    $ret =~ s/\n/\n$$: /g;
-    warn "$$: $ret";
-    return 1;
-
-
     my $api=POE::API::Peek->new();
     my @queue = $api->event_queue_dump();
     
@@ -805,7 +886,7 @@ sub __peek
         $ret.="Keepalive " unless $verbose;
         $ret.="Sessions: \n";
         my $ses;
-        foreach my $session ($api->session_list) {  
+        foreach my $session ( sort { $a->ID <=> $b->ID } $api->session_list) {  
             my $ref=0;
             $ses='';
 
@@ -840,7 +921,7 @@ sub __peek
                 $ref += $q1;
             }
 
-            my $q1 = $events->{ $session->ID }{destination};
+            $q1 = $events->{ $session->ID }{destination};
             if( $q1 ) {
                 $ret.="\t\tEvent destination count: $q1 (Stay alive)\n";
                 $ref += $q1;
@@ -900,10 +981,10 @@ POE::Component::IKC::Server - POE Inter-kernel Communication server
 
     use POE;
     use POE::Component::IKC::Server;
-    create_ikc_server(
+    POE::Component::IKC::Server->spawn(
         ip=>$ip, 
         port=>$port,
-        name=>'Server',);
+        name=>'Server');
     ...
     $poe_kernel->run();
 
@@ -918,7 +999,13 @@ identical.
 
 =head2 C<create_ikc_server>
 
-This function initiates all the work of building the IKC server.  
+Syntatic sugar for POE::Component::IKC::Server->spawn.
+
+=head1 CLASS METHODS
+
+=head2 C<spawn>
+
+This methods initiates all the work of building the IKC server. 
 Parameters are :
 
 =over 3
@@ -930,7 +1017,14 @@ Address to listen on.  Can be a doted-quad ('127.0.0.1') or a host name
 
 =item C<port>
 
-Port to listen on.  Can be numeric (80) or a service ('http').
+Port to listen on.  Can be numeric (80) or a service ('http').  If
+undefined, will default to 603.  
+If you set the port to 0, a random port will be chosen and C<spawn> will
+return the port number.
+
+    my $port = POE::Component::IKC::Server->spawn( port => 0 );
+    warn "Listeing on port $port";
+
 
 =item C<unix>
 
@@ -972,8 +1066,31 @@ connections are serviced concurrently, because there's no way to know when
 we have finished a request.  Defaults to 1 (ie, one connection per
 child).
 
+=item C<concurrency>
+
+Number of simultaneous connected clients allowed.
+Defaults to 0 (unlimited).  
+
+Note that this is per-IKC::Server instance;  if you have several ways of
+connecting to a give IKC server (for example, both an TCP/IP port and unix
+pipe), they will not share the conncurrent connection count.
+
+=item C<protocol>
+
+Which IKC negociation protocol to use.  The original protocol (C<IKC>) was
+synchronous and slow.  The new protocol (C<IKC0>) sends all information at
+once.  IKC0 will degrade gracefully to IKC, if the client and server don't
+match.
+
+Default is IKC0.
+
+
 =back
 
+C<POE::Component::IKC::Server::spawn> returns C<undef()>, unless you specify
+a L</port>=0, in which case, C<spawn> returns the port that was chosen.
+
+
 =head1 EVENTS
 
 =head2 shutdown
@@ -994,7 +1111,7 @@ Philip Gwyn, <perl-ikc at pied.nu>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 1999-2008 by Philip Gwyn.  All rights reserved.
+Copyright 1999-2011 by Philip Gwyn.  All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
